/*
 Need to implement a tab view, and on the second view, show the output of the system command.
 Change the Window title between each tab view, and keep the bottom controls avialable for each 
 tab view. Use popen() and spawn_thread to do the actual work of spawning doing the system process.
 
 (6177) marcone: popen("command","r") will run "command", and return a FILE* that you can use to read from it's stdout
*/
//system headers
#include <Application.h>
#include <Message.h>
#include <Roster.h>
#include <stdio.h>
#include <string.h>
#include <View.h>
#include <Font.h>
#include <Path.h>
#include <File.h>
#include <Rect.h>
#include <FindDirectory.h>

// project headers
#include "Colors.h"
#include "Messages.h"

// liblayout headers
#include "MWindow.h"
#include "VGroup.h"
#include "MBorder.h"
#include "MScrollView.h"
#include "Space.h"
#include "MTextControl.h"
#include "VGroup.h"
#include "HGroup.h"
#include "MButton.h"
#include "MBorder.h"
#include "MTextView.h"
#include "TabGroup.h"
#include "MCheckBox.h"

// App header
#include "TrackWatcher.h"


// function prototypes
void *getplugin(void **data,const char *name, const char *, uint32, plugin_info * );
void destroyplugin(void*,void *_loader);
status_t show(void*);
void hide(void*);
BView *configure(BMessage *config);
void setconfig(void*,BMessage *config);
void getconfig(void*,BMessage *config);
void reloadfont(BFont *font[]);

// window objects
MButton *saveb;
MButton *closeb;
MTextControl *exeline;
MTextControl *ascline;
LoggerWindow *gWinPointer;
MTextView *gtvPointer;
MTextView *gcvPointer;
Tasker *gWorkerPtr;

const rgb_color RowGrey = {242,242,242,255};
const rgb_color RowWhite = {255,255,255,255};
const rgb_color BGDefault = {236,236,236,255};
const rgb_color Page2 = {150,150,150,255};
const rgb_color RunGreen = {0,227,0,255};

const char *PLUGINID="sp-trackwatcher";

const char *exepath;
const char *exemsg;
const char *running;
char songname[1024];
char lm[1024];
char logmes[1024];
char response[2048];
const char *t_response;
char ascmesgbuf[1024];
bool firstloop = true;
bool doexecute = true;
bool gStartMinimized;
bool logmessage;
bool hascontrol = false;
bool change_my_punk_ass = false;
int32 gCurrentID;
rgb_color gViewcolor;
PlaylistPtr gCurrentPlaylist;

//plugin description configuration. Allow
static plugin_descriptor plugin={ 
	PLUGIN_DESCRIPTOR_MAGIC,PLUGIN_DESCRIPTOR_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,
	"Track Watcher",
	"By Zach Dykstra.\n\n"
	"Execute a script or program when"
	" the track changes. \n\n",
	NULL,
	&configure,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_INTERFACE_MAGIC, PLUGIN_INTERFACE_VERSION,
	show,
	hide,
	setconfig,
	getconfig
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}

// instantiate the plugin
void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info )
{
	LoggerWindow *win=new LoggerWindow(info->controller);
	gWinPointer = win;
	info->controller->AddWindow(win,FILTER_HOTKEYS); // let SoundPlay handle some hotkeys for this window
	*data=win;
	return &plugin_ops;
}

// destroy the plugin -- wonkified?
void destroyplugin(void *,void *data)
{
	LoggerWindow *win=(LoggerWindow*)data;
	win->Lock();
	win->Quit();
}

// show 
status_t show(void *data)
{
	LoggerWindow *win=(LoggerWindow*)data;
	gWinPointer = win;
	win->Show();
	return B_OK;
}

// hide
void hide(void *data)
{
	LoggerWindow *win=(LoggerWindow*)data;
	gWinPointer = win;
	win->Hide();
}

LoggerWindow::LoggerWindow(SoundPlayController *controller)
	: MWindow(BRect(100,100,300,350),"Track Watcher",B_TITLED_WINDOW,B_NORMAL_WINDOW_FEEL,B_ASYNCHRONOUS_CONTROLS)
{
	ctrl=controller;
	AddChild
	(
		new VGroup
		(
			new MBorder
			(
				M_ETCHED_BORDER,5,"",
				new VGroup
				(
					new TabGroup
					(
						"Song Listing",
						new MScrollView
						(
							tv=new MTextView(),
							false,
							true			
			 			),
						"Command Output",
						new MScrollView
						(
					 		cv=new MTextView(),
							false,
							true
						),0
					),
					new Space
					(
						minimax(5,5,1,1)
					),
					new HGroup
					(
						new MButton("Toggle Execution", new BMessage(M_TOG_EXE),NULL,minimax(-1,-1,-1,500)),
						new MButton("Hide",new BMessage(M_HIDE_LOG),NULL,minimax(-1,-1,-1,500)),0
					),0
				)
			),0
		),0
	);

	gtvPointer = tv; 
	gcvPointer = cv;
	// set flags for the text views
	tv->SetStylable(true);
	tv->MakeEditable(false);
	tv->MakeSelectable(false);
	cv->SetStylable(true);
	cv->MakeEditable(false);
	cv->MakeSelectable(false);
}

LoggerWindow::~LoggerWindow()
{
	gWorkerPtr->PostMessage(B_QUIT_REQUESTED);
}

void LoggerWindow::Show() // we only care about track 0, we only want tw to watch it's track changes
{
 	worker = new Tasker(gWinPointer,"backgrounder",B_NORMAL_PRIORITY,B_LOOPER_PORT_DEFAULT_CAPACITY);
 	gWorkerPtr = worker;
	BMessage mes(T_EXECUTE_LINE);
	mes.AddString("execute",exepath);
	gWorkerPtr->PostMessage(&mes);
	ctrl->Lock();
	LogMessage("Starting up Track Watcher\n", C_SET_LOG,1);
	ctrl->AddListener(this);
	PlaylistPtr playlist=ctrl->PlaylistAt(0);
	if(playlist->Lock())
	{	
		gCurrentPlaylist = playlist;
		playlist->AddListener(this);
		system("rm -f /boot/home/config/settings/tw_info");
		gCurrentID = playlist->CurrentID();
		hascontrol=true;
		ExecuteAndLog();
		playlist->Unlock();
	}
	ctrl->Unlock();
	MWindow::Show();
}

void LoggerWindow::LogMessage(const char *message, uint32 colorid, int32 page)
{
	char string[1000];
	sprintf(string,message);
	text_run_array tarray;
	// find the color to set the text
	switch (colorid)
	{
		case C_SET_RUN:
		{
  	  		text_run log[1] = {0, be_plain_font, RunGreen};
	  		tarray.count = 1;
	  		tarray.runs = log;
	 	}
	 	break;
	 	
		case C_SET_LOG:
	 	{
  	  		text_run log[1] = {0, be_plain_font, BeFocusBlue};
	  		tarray.count = 1;
	  		tarray.runs = log;
	 	}
	 	break;
	 	 
	 	case C_SET_SONG:
	 	{
		  	text_run song[1] = {0, be_plain_font, Black};
		  	tarray.count = 1;
		  	tarray.runs = song;
	 	}
	 	break;
	}
	// insert the text, scroll to it
	if (page == 1)
	{
		tv->Insert(string,&tarray);
		tv->ScrollToSelection();
	}
	if (page == 2)
	{
		cv->Insert(string,&tarray);
		cv->ScrollToSelection();
	}
}

void LoggerWindow::ExecuteAndLog()
{
	// is the file actually valid? If so, print to the window, write info to the file, and execute system command
	if (hascontrol == true)
	{
		ctrl->Lock();
		PlaylistPtr playlist = ctrl->PlaylistAt(0);
		playlist->Lock();
		if (playlist->CurrentName() != NULL)
		{
			strcpy(songname,playlist->CurrentName());
			strcat(songname,"\n");
			LogMessage(songname,C_SET_SONG,1);
			if (doexecute == true)
			{
	    		// EVIL way to do it
	    		char echo[1024];
	    		sprintf(echo,"echo \"%s;%d \"%s",playlist->CurrentName(),playlist->CurrentID(),"> /boot/home/config/settings/tw_info");
				system(echo);
				BMessage mes(T_RUN_COMMAND);
				gWorkerPtr->PostMessage(&mes);
			}
		}

		playlist->Unlock();
		ctrl->Unlock();
	}
}


void LoggerWindow::MessageReceived(BMessage *mes)
{
	switch(mes->what)
	{	
		case T_RUN_RESPONSE:
		{
			LogMessage(mes->FindString("response"),C_SET_RUN,2);
			
			// put the comment in the log window? 
			if (logmessage == true)
			{
				strcpy(logmes,exemsg);
				strcat(logmes,"\n");
				LogMessage(logmes,C_SET_LOG,1);
			}
		}
		break;
		
		case PLAYLIST_ADD:
		{
			if (change_my_punk_ass == true)
			{
				ctrl->Lock();
				if (hascontrol == false)
				{
					change_my_punk_ass = false;
					PlaylistPtr playlist = ctrl->PlaylistAt(0);
					playlist->Lock();
					gCurrentPlaylist = playlist;
					gCurrentID = playlist->CurrentID();
					hascontrol = true;
					gCurrentPlaylist = playlist;
   	  	   			if (gCurrentID != playlist->CurrentID())
   	        		{
  // 	         			gCurrentID = playlist->CurrentID();
             			ExecuteAndLog();
            		}
					playlist->Unlock();
				}
				ctrl->Unlock();
			}
		}
		break;

		case PLAYLIST_EMPTY:
		{
			if (gCurrentPlaylist->IsValid())
			{
				
			}
			hascontrol = false;
			change_my_punk_ass = true;
			
		}
		
		break;
	
		// see if the playlist we are watching looses controls, if so, monitor the new one at 0
		case PLAYLIST_LOST_CONTROLS:
    	{
			hascontrol = false;
			ctrl->Lock();
			
			if (ctrl->CountPlaylists() > 0)
        	{
				PlaylistPtr playlist = ctrl->PlaylistAt(0);
				if (playlist->Lock())
				{
					hascontrol = true;
					playlist->AddListener(this);
					gCurrentPlaylist = playlist;
   	  	   			if (gCurrentID != playlist->CurrentID())
   	        		{
   	         			gCurrentID = playlist->CurrentID();
             			ExecuteAndLog();
            		}
				playlist->Unlock();
				}
			}
			ctrl->Unlock();
		}
		break;

    	// adding a new playlist? are we already monitoring one? if so, ignore it
		case CONTROLLER_ADD:
		{
			int32 id;
			ctrl->Lock();
			if (hascontrol == false)
			{
				if(B_OK==mes->FindInt32("Playlist",&id))
				{
					hascontrol = true;
					PlaylistPtr playlist(id);
					if (playlist->Lock())
					{
						playlist->AddListener(this); 
						gCurrentPlaylist = playlist;
						if (gCurrentID != playlist->CurrentID())
						{
							gCurrentID = playlist->CurrentID();
							ExecuteAndLog();
						}
					playlist->Unlock();
					}
				}
			}
			ctrl->Unlock();
 		}
		break;
     
   		// the current song in track 0 changed
		case PLAYLIST_CURRENT:
		{
			if (hascontrol == true)
			{
				ctrl->Lock();
				PlaylistPtr playlist = ctrl->PlaylistAt(0);
				ctrl->Unlock();
				playlist->Lock();
				if (gCurrentID != playlist->CurrentID())
   	    		{
					gCurrentID = playlist->CurrentID();
					ExecuteAndLog();
				}
				playlist->Unlock(); 	
			}
		}
   		break;
  
  		// toggle execution button pushed
		case M_TOG_EXE:
    	{
			bool tempex = doexecute;  
			if (tempex == true)
       		{
				doexecute = false;
				LogMessage("Command execution halted \n",C_SET_LOG,1);
			}
			if (tempex == false)
			{
				doexecute = true;
				LogMessage("Command execution started \n",C_SET_LOG,1);
			}
		}
		break;
	
		// hide the window
		case M_HIDE_LOG:
		{
			gWinPointer->Lock();
			gWinPointer->Minimize(true);
			gWinPointer->Unlock();
		}
		break;
	} 
}

bool LoggerWindow::QuitRequested()
{
	ctrl->DisableInterface(PLUGINID);
	return false;
}

BView* configure(BMessage *config)
{
	return new ConfigView(config);
}


void setconfig(void *data,BMessage *config)
{
	LoggerWindow *win=(LoggerWindow*)data;
 	gWinPointer = win;
 	// set instance variables
 	doexecute = true;
 	
 	// should the window be minimized or re-shown?
	gStartMinimized = config->FindBool("hidebydefault");
	if (true == gStartMinimized)
	{
		win->Minimize(true);
	}
	if (false == gStartMinimized)
	{
		win->Minimize(false);
	}
		
	// store the windows size and position
	BRect rect;
	if(B_OK==config->FindRect("frame",&rect))
	{
		win->MoveTo(rect.LeftTop());
		win->ResizeTo(rect.Width(),rect.Height());
	}
	
	// set the variables for execute command and associated message
	if(!config->HasString("execute"))
	{
		config->AddString("execute","default");
	}
	
	
	if (B_OK==config->FindString("execute",&exepath))
	{
		BMessage mes(T_EXECUTE_LINE);
		mes.AddString("execute",exepath);
		gWorkerPtr->PostMessage(&mes);
	}
	
	// go back to setting other shit
	config->FindString("exemsg",&exemsg);
	logmessage = config->FindBool("logexmesg");
    int32 precolor;
    
    // set the song listing view color
    if (config->HasInt32("tvcolor")==true)
	{
		config->FindInt32("tvcolor",&precolor);
		rgb_color setcolor = *(rgb_color*)(&precolor);
		gWinPointer->Lock();
		gtvPointer->SetHighColor(Black);
		gtvPointer->SetLowColor(setcolor);
		gtvPointer->SetViewColor(setcolor);
		gWinPointer->Unlock();
	}
	// else we set it to the default color
	else
	{
		gWinPointer->Lock();
		gtvPointer->SetHighColor(Black);
		gtvPointer->SetLowColor(BGDefault);
		gtvPointer->SetViewColor(BGDefault);
		gWinPointer->Unlock();
	}
	// set the command output view color
	if (config->HasInt32("cvcolor")==true)
	{
		config->FindInt32("cvcolor",&precolor);
		rgb_color setcolor = *(rgb_color*)(&precolor);
		gWinPointer->Lock();
		gcvPointer->SetHighColor(RunGreen);
		gcvPointer->SetLowColor(setcolor);
		gcvPointer->SetViewColor(setcolor);
		gWinPointer->Unlock();
	}
	else
	{
		gWinPointer->Lock();
		gcvPointer->SetHighColor(RunGreen);
		gcvPointer->SetLowColor(Page2);
		gcvPointer->SetViewColor(Page2);
		gWinPointer->Unlock();
	}
	
}

void getconfig(void *data,BMessage *config)
{
	LoggerWindow *win=(LoggerWindow*)data;
	gWinPointer = win;

	// make an entry to store the window size/position
	BRect rect;
	if(B_OK==config->FindRect("frame",&rect))
	{
		config->ReplaceRect("frame",win->Frame());
	}
	else
	{
		config->AddRect("frame",win->Frame());
	}
	
	// find the color of the text view
	gWinPointer->Lock();
	rgb_color temp = gtvPointer->ViewColor();
	gWinPointer->Unlock();
	
	// do we already have an entry? if not, make a new one, and store it
	if (config->HasInt32("tvcolor")==true)
	{
		config->ReplaceInt32("tvcolor",*(uint32*)(&temp));
	}
	else
	{ 
		config->AddInt32("tvcolor",*(uint32*)(&temp));
	}
	
	// second text view color
	gWinPointer->Lock();
	rgb_color temp2 = gcvPointer->ViewColor();
	gWinPointer->Unlock();
	
	if (config->HasInt32("cvcolor")==true)
	{
		config->ReplaceInt32("cvcolor",*(uint32*)(&temp2));
	}
	else
	{ 
		config->AddInt32("cvcolor",*(uint32*)(&temp2));
	}
	
	// find the execute and log strings, store 'em
	
	if (B_OK==config->FindString("execute",&exepath))
	{
		BMessage mes(T_EXECUTE_LINE);
		mes.AddString("execute",exepath);
		gWorkerPtr->PostMessage(&mes);
	}
	
	config->FindString("exemsg",&exemsg);
	
	// find the show message bool
	logmessage = config->FindBool("logexmesg");
}

ConfigView::ConfigView(BMessage *_config)
	: VGroup((MView*)NULL)
{
	config=_config;
	
	// set the default checkboxs if they don't exsist
	if(!config->HasBool("hidebydefault"))
	{
		config->AddBool("hidebydefault",false);
	}
	if(!config->HasBool("logexmesg"))
	{
		config->AddBool("logexmesg",false);
	}
 
	// the first group - execute/log message options 
	AddChild
	(
		new VGroup
		(
			exeline=new MTextControl("Execute ",""),
			ascline=new MTextControl("Message",""),
			new MCheckBox("Hide log window on plugin startup",new BMessage(M_HIDE_DEFAULT),this,config->FindBool("hidebydefault")),
			new MCheckBox("Show extra message in log window",new BMessage(M_LOG_MSG),this,config->FindBool("logexmesg")),0
		),0
	);
	
	// save the stuff, close the window?
	AddChild
	(	
		new HGroup
		(
			closeb = new MButton("Close", new BMessage(B_QUIT_REQUESTED),NULL,minimax(-1,-1,-1,-1)),
			saveb = new MButton("Save", new BMessage(M_SAVEC),this,minimax(-1,-1,-1,-1)),0
		),0
	);

	// set the text in the views, if it has it
	config->FindString("execute",&exepath);
	exeline->SetText(exepath);
	config->FindString("exemsg",&exemsg);
	ascline->SetText(exemsg);
	saveb->MakeDefault(true);	
}

ConfigView::~ConfigView()
{

}

void ConfigView::MessageReceived(BMessage *mes)
{
	switch(mes->what)
	{
		// save button was pressed
		case M_SAVEC:
		{		
			config->AddString("execute",exeline->Text());
			config->AddString("exemsg",ascline->Text());
	 		config->ReplaceString("execute", exeline->Text());
	 		config->ReplaceString("exemsg", ascline->Text());
	 		exepath = exeline->Text();
	 		exemsg = ascline->Text();
		}
		break;
		
		// the hide window by default check box was pressed
		case M_HIDE_DEFAULT:
		{
			config->ReplaceBool("hidebydefault",mes->FindInt32("be:value"));
		}
		break;
			
		case M_LOG_MSG:
		{
			config->ReplaceBool("logexmesg",mes->FindInt32("be:value"));
		}
		break;
	}
}

int main()
{
	BApplication app("application/x-vnd.md-trackwatcher");
	BMessage msg(ENABLE_INTERFACE);
	msg.AddString("interface",PLUGINID);
	be_roster->Launch("application/x-vnd.marcone-soundplay",&msg);
	return 0;
}